// go.h - GAMS movement 
//
// Nov-2004 by Solo, adapted from Serge 
// Nov-2007 by Solo, change in retry logic
//


// 


#ifndef _GO_H
#define _GO_H

#include <math.h>   // pow
#include <stdlib.h> // abs
#include <unistd.h> // usleep
#include <stdint.h>

#include "le20c.h" // QBUS-SUMMA host
/////// /////
#include <time.h>
#include <sys/time.h>
//////  /////

// 0.092 or 3/32 - mm/summa
// 8.35          - summa/datchik
// 0.77          - mm/datchik



#define XY_DISABLE 0
#define X_ENABLE   1
#define Y_ENABLE   2
#define XY_ENABLE  3



#define X_MIN 1700
#define X_MAX 22330
#define Y_MIN 3100
#define Y_MAX 18750



// X, Y coordinates
typedef struct
{
  int X, Y;
} XY;

// Counter coordinates
typedef struct
{
  int column;
  int row;
  int quarter;
} CRQ;

//const int GO_X0 = -756+290; // up to run33
//const int GO_X0 = -1770;      // from run34, new beam line
const int GO_X0 = -1410; // from run35, after X encoder repair
//const int GO_Y0 = 21015; // up to run35
const int GO_Y0 = 20835; // from run35, new beam line 

const float GO_dX[5] = { 0, -21.5, -21.5, 21.5, 21.5 };
const float GO_dY[5] = { 0, 21.5, -21.5, 21.5, -21.5 };

//const int GO_miss_X = 10; // encoder units
//const int GO_miss_Y = 30; // encoder units
const int GO_miss_X = 30; // encoder units, run35
const int GO_miss_Y = 80; // encoder units, run35

const int GO_timeout    = 120;   // s
const int GO_stick_wait = 1;    // s


class GO
{
 public:
  // constructor
  GO(LE20C* host,int nx=6, int ny=5, FILE* file=0) {
    h = host;
    X_n = nx;
    Y_n = ny;
    f = file;
  }

  // destructor
  ~GO() {}

  // useful methods

  // convert Column/Row/Quarter to X/Y
  void crq2xy (CRQ* from, XY* to) {
    to->X = (int)((86.0*from->column + GO_dX[from->quarter])*32.0/3.0) + GO_X0;
    to->Y = (int)((-86.0*from->row + GO_dY[from->quarter])*32.0/3.0) + GO_Y0;
  }

  // convert X/Y to Column/Row/Quarter
  void xy2crq (XY* from, CRQ* to) {
    double D;
    double minD = 1.0e6;
    int minQ=0;
    XY approx;
    
    // Calculating nearest colomn and row
    to->column = (int)(((from->X - GO_X0)*3.0/32.0)/86.0 + 0.5);
    to->row = (int)(0.5 - ((from->Y - GO_Y0)*3.0/32.0)/86.0);

    // Looking for the nearest quarter
    for (to->quarter = 0; to->quarter < 5; to->quarter++) {
      crq2xy (to, &approx);
      D = pow ((float)approx.X - (float)from->X, 2) +
	pow ((float)approx.Y - (float)from->Y, 2);
      if (D < minD) {
	minD = D;
	minQ = to->quarter;
      }
    } 

    to->quarter = minQ;
  };

  // get/set coordinates
  void set_X (short coord) { h->write (X_n, 0, coord); };
  void set_Y (short coord) { h->write (Y_n, 0, coord); };
  int  get_X () { return h->function (X_n, 1, 2); };
  int  get_Y () { return h->function (Y_n, 1, 2); };
  void get_XY (XY* pos) { pos->X = get_X(); pos->Y = get_Y(); };

  int get_sek_time() {
    struct timeval tv;
    struct timezone tz;
    gettimeofday( &tv,&tz );
    return tv.tv_sec ;
  }
  int64_t get_mks_time() {
    struct timeval tv;
    struct timezone tz;
    gettimeofday( &tv,&tz );
    return ((int64_t)1000000)*tv.tv_sec + tv.tv_usec;
  }


  // return true if moving
  int X_moving () { return (h->function (X_n, 2, 1) & 0x8000) == 0; };
  int Y_moving () { return (h->function (Y_n, 2, 1) & 0x8000) == 0; };


 

  // print current status

  void print () {
    const char* str[2] = { "no action", "moving" };
    XY xy;
    CRQ crq;

    get_XY (&xy);
    xy2crq (&xy, &crq);

    //    printf ("GO status:\n");
    printf ("X axis: status = %s, coord = %d\n", str[X_moving ()], xy.X);
    printf ("Y axis: status = %s, coord = %d\n", str[Y_moving ()], xy.Y);
    printf ("Closest position: column = %d, row = %d, quarter = %d\n",
	    crq.column, crq.row, crq.quarter);
  };
  //////////////////////// my //// get new position
  XY posit() {
    XY ddd;
    ddd.X=get_X();
    ddd.Y=get_Y();
    return ddd;
  }; 


  // move GAMS to new position
  int move (XY* dest, int ax=XY_ENABLE) {
    short last_coord;
    int s;

    // start X movement
    if ((ax & X_ENABLE) != 0) {
      X_moving ();

      if (abs (get_X () - dest->X) > GO_miss_X)
	{
	  last_coord = get_X ();
          fprintf (f,"Moving X from %d to %d...\n", last_coord, dest->X);
          set_X (dest->X);
	  sleep (GO_stick_wait*3);
          if (abs (get_X () - dest->X) >= abs (last_coord - dest->X))
	    {
	      fprintf (f,"Alas: X movement stuck\n");
	      h->Z ();
	    }
        }
      else fprintf (f,"X coordinate is OK\n");
    }

    // start Y movement
    if ((ax & Y_ENABLE) != 0)
      {
	Y_moving ();

	if (abs (get_Y () - dest->Y) > GO_miss_Y)
	  {
	    last_coord = get_Y ();
	    fprintf (f,"Moving Y from %d to %d...\n", last_coord, dest->Y);
	    set_Y (dest->Y);
	    sleep (GO_stick_wait);
	    if (abs (get_Y () - dest->Y) >= abs (last_coord - dest->Y))
	      {
		fprintf (f,"Alas: Y movement stuck\n");
		h->Z ();
	      }
	 
	  }
	else
	 fprintf (f,"Y coordinate is OK\n");
      }

    ///////////////   ///////////////
    int last_coord_Y=get_Y(); 
    int last_coord_Y2=get_Y();
    int last_coord_X=get_X(); 
    int last_coord_X2=get_X();
    int X_moved,Y_moved, X_moved2,Y_moved2, now_getX,now_getY;
    int res_X_moved=0, res_Y_moved=0;
    int64_t sleep_time_mks;
    int64_t sleep_time_mks2;
    int timeoff=0;
    int myzero;
    XY flag_check_miss_dest;
    XY miss_appointment;
    miss_appointment.X=0;
    miss_appointment.Y=0;

    int time_from_start=get_sek_time();


    int64_t nowtime_mks=get_mks_time();
    int64_t nowtime_mks2=get_mks_time();



    XY jump;
    jump.X=0;
    jump.Y=0;

    XY coord_jump;


    int64_t time_loc_jumpX=get_mks_time();
    int64_t time_loc_jumpY=get_mks_time();
    int64_t time_loc_jump_now=0;

    while(X_moving() or Y_moving() ) {

// if gams moved out of border - then stop
if (get_X()!=0  or last_coord_X!=0 ) 
 { if (get_X()>X_MAX or get_X()<X_MIN ) 
   { fprintf (f,"The movement have moved outside the border on an axis X\n"); 
     stop();
   }
}
if (get_Y()!=0 or last_coord_Y!=0 )
{  if (get_Y()<Y_MIN or get_Y()>Y_MAX )
    {
     fprintf (f,"The movement have moved outside the border on an axis Y\n");
     stop();
    }
}

#if 1
      time_loc_jump_now=get_mks_time();
    if(get_X()!=0 and  jump.X!=0 and (abs(time_loc_jump_now - time_loc_jumpX)>2000000))   {
      jump.X=0;
      time_loc_jumpX=get_mks_time();
    }

    if(get_Y()!=0 and  jump.Y!=0 and (abs(time_loc_jump_now - time_loc_jumpY)>2000000))   {
      jump.Y=0;
      time_loc_jumpY=get_mks_time();
    }





#endif
    now_getX=get_X();
    now_getY=get_Y();
    X_moved=(now_getX-last_coord_X);
    Y_moved=(now_getY-last_coord_Y);
    X_moved2=(now_getX-last_coord_X2);
    Y_moved2=(now_getY-last_coord_Y2);
    sleep_time_mks=( (int64_t)(get_mks_time())- ((int64_t)nowtime_mks));
    sleep_time_mks2=( (int64_t)(get_mks_time())- ((int64_t)nowtime_mks2) );

#if 1


//  ????????????
    if(myzero==get_X())
      {printf("Attention: get_X=0\n"); }
    if(myzero==get_Y())
      {printf("Attention: get_Y=0\n"); }
//  ????????????


    if (flag_check_miss_dest.X==1)
      {
	if (now_getX-last_coord_X<0 and jump.X==0)
	  { do { jump.X++; } while(jump.X<1);
	  }
	if (now_getX-last_coord_X>0 and jump.X==1)
	  {do  {jump.X++; } while (jump.X<2);
	  }
	if (now_getX-last_coord_X<0 and jump.X==2)
	  {
	    fprintf(f,"1 Attention!!! the movenment is jumping on axis X.\n");
	    h->Z();
	    return 0;
	  }
      }
    if (flag_check_miss_dest.X==-1)
      {
	if (now_getX-last_coord_X>0 and jump.X==0)
	  { do { jump.X++;}  while(jump.X<1);
	  }
	if (now_getX-last_coord_X<0 and jump.X==1)
	  {do  {jump.X++; } while (jump.X<2);
	  }
	if (now_getX-last_coord_X>0 and jump.X==2)
	  {
	    fprintf(f,"2 Attention!!! the movenment is jumping on axis X.\n");
	    h->Z();
	    return 0;
	  }
      }

    if (flag_check_miss_dest.Y==1)
      {
	if (now_getY-last_coord_Y<0 and jump.Y==0)
	  { do { jump.Y++; } while(jump.Y<1);
	  }
	if (now_getY-last_coord_Y>0 and jump.Y==1)
	  {do  {jump.Y++; } while (jump.Y<2);
	  }
	if (now_getY-last_coord_Y<0 and jump.Y==2)
	  {
	   fprintf(f,"1 Attention!!! the movenment is jumping on axis Y.\n");
	    h->Z();
	    return 0;
	  }
      }
    if (flag_check_miss_dest.Y==-1)
      {
	if (now_getY-last_coord_Y>0 and jump.Y==0)
	  { do {jump.Y++;} while(jump.Y<1);
	  }
	if (now_getY-last_coord_Y<0 and jump.Y==1)
	  {do  {jump.Y++; } while (jump.Y<2);
	  }
	if (now_getY-last_coord_Y>0 and jump.Y==2)
	  {
	    fprintf(f,"2 Attention!!! the movenment is jumping on axis Y.\n");
	    h->Z();
	    return 0;
	  }
      }
#endif



#if 1
    // printing values on display and in files while moving
    if ( sleep_time_mks2>(int64_t)1000000 ) 
      {
	if (f)  {
     	if (Y_moved2==0 && X_moved2==0) 
	{
	  timeoff++;
	  if (timeoff>=2) { fprintf (f,"Alarm: movement is of. Protection has worked\n"); h->Z (); }
	}
		}    
	nowtime_mks2=get_mks_time();
	last_coord_X2=now_getX;
	last_coord_Y2=now_getY;


      }
#endif





    if (f) {
      if ( abs(sleep_time_mks)>((int64_t)20000)  ) 
	{

	  if ( X_moved>10 || Y_moved>10 || X_moved<-10 || Y_moved<-10 || sleep_time_mks>200000) 

	    {  fprintf(f, "Attention! Big displacement or delay between cycles\n----------\nX is: %d moved: %d    Y is: %d moved: %d  time : %lld \tlast_coord_X:%d\n----------\n", now_getX,X_moved,now_getY,Y_moved,sleep_time_mks,last_coord_X );
	
	    }
/*
	  else
	    { 
	      fprintf(f, "X is: %d moved: %d    Y is: %d moved: %d  time : %lld \tlast_coord_X:%d\n",now_getX,X_moved, now_getY,Y_moved, sleep_time_mks,last_coord_X );
	    }
*/

	 if (X_moved>10 or X_moved<-10)
        { res_X_moved= res_X_moved+X_moved; }
        if (Y_moved>10 or Y_moved<-10)
        { res_Y_moved= res_Y_moved+Y_moved; }

	  
	   nowtime_mks=get_mks_time();
	  last_coord_X=now_getX;
	  last_coord_Y=now_getY;
	}
    }

  }




  sleep(GO_stick_wait);
 fprintf(f,"Operation complete\n");

if(res_X_moved!=0)
   { fprintf (f,"The sum of jumps on an axis X: %d\n",res_X_moved); }
if(res_Y_moved!=0)
   {fprintf (f,"The sum of jumps on an axis Y: %d\n",res_Y_moved); }


  if (((ax & X_ENABLE) != 0) and (abs (get_X () - dest->X) > GO_miss_X))
    { fprintf (f,"Alas: X movement missed\n");
    }
if (((ax & Y_ENABLE) != 0) and (abs (get_Y () - dest->Y) > GO_miss_Y))
    { fprintf (f,"Alas: Y movement missed\n");
    }


  h->Z ();
  print();

  return true;
};

// move to destination coordinates
int move(CRQ *dest, int ax=XY_ENABLE) {
  XY dest_xy;
  crq2xy(dest, &dest_xy);
  return move(&dest_xy, ax);
}

// stop the movement
int stop() {
  h->Z();
  return 0;
}

// write to file

void write_to_f( char *text )  {
 if (f)
  fprintf (f,"%s",*text);
}



private:
LE20C *h; // SUMMA host
int X_n;  // X module number
int Y_n;  // Y module number
FILE *f;

};

#endif
