// le90_dim.cc - dim server for SUMMA LE90M control unit
//
// Nov-2008 by Solo
//

#include <stdio.h>
#include <string.h>
#include <time.h>
#include "le20c.h"
#include "le90m.h"

#include "dis.hxx"

int main(int argc, char **argv)
{
  time_t tt;
  LE90MDATA le90m_data;
  struct {
    int n;
    int crate;
    char qname[80];
  } le90m_setup;

  if(argc<4) {
    printf("Usage: le90m_dim qbus crate N \n");
    printf("       qbus  = 0-2  - qbus number or /dev/xxx \n");
    printf("       crate = 0-15 - summa crate number \n");
    printf("       N     = module number in crate \n");
    return 0;
  }

  strncpy(le90m_setup.qname,argv[1],sizeof(le90m_setup.qname));
  if(argv[1][0]=='0') strcpy(le90m_setup.qname,"/dev/qbus0");
  if(argv[1][0]=='1') strcpy(le90m_setup.qname,"/dev/qbus1");
  if(argv[1][0]=='2') strcpy(le90m_setup.qname,"/dev/qbus2");

  sscanf(argv[2],"%d",&le90m_setup.crate);
  if(le90m_setup.crate<0 || le90m_setup.crate>15) { 
    printf("SUMMA crate %d is out of [0-15] range\n",le90m_setup.crate); 
    return -1;
  }

  sscanf(argv[3],"%d",&le90m_setup.n);
  if(le90m_setup.n<0 || le90m_setup.n>24) { 
    printf("SUMMA module number %d is out of [0-24] range\n",le90m_setup.n); 
    return -1;
  }

  QBUS qb(le90m_setup.qname);
  if(qb.node()<0) return -1;

  LE20C su(&qb, le90m_setup.crate);

  LE90M le(&su, le90m_setup.n);

  DimService setup("BEC_SYNCH/LE90M_SETUP","I:2;C",&le90m_setup,sizeof(le90m_setup));
  DimService data("BEC_SYNCH/LE90M_DATA","S:11",&le90m_data,sizeof(le90m_data));
  DimCommand cmd("BEC_SYNCH/LE90M_CMD","C");

  DimServer::start("BEC_SYNCH");

  tt = time(NULL);
  printf("Started BEC_SYNCH DIM service... %s",ctime(&tt));

  while (true) {
    le.read(&le90m_data);
    data.updateService();
    while (cmd.getNext()) {
      printf("Command: %s\n",cmd.getString());
      if(strcmp(cmd.getString(),"enable")==0) {
	le.enable();
      } 
      else if(strcmp(cmd.getString(),"disable")==0) {
	le.disable();
      } 
      else {
	printf("Unknown command: %s\n",cmd.getString());
      }
    }
    sleep(10);
  }

  return 0;
}
