/******************************************************************************
    GAMS - library for GAMS movement in VES experiment

    Copyright (C) 1997, Alexandre Solodkov
    Portions Copyright (C) 1997, Sergei Kopikov
    Institute for High Energy Physics (IHEP), Protvino, Russia

    This program is free software; you can redistribute it and/or modify
    it, but you may not sell it.

    This program is distributed in the hope that it will be useful,
    but without any warranty; without even the implied warranty of
    merchantability or fitness for a particular purpose.

	Alexandre Solodkov:     solodkov@mx.ihep.su

    Returns from subroutines:
	0   - success
	1   - wrong host name
	2   - unable to reach it - local host or first router is down
	3   - connection closed
	4   - problems during close
	5   - timeout
	6   -
	7   - output buffer is full
      < 0   - errors on server

******************************************************************************/

#define DEBUG

//#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

# include "ether.h"

int SocketId;

/*******************************************************************/

int sconnect (int argc, char **argv)
{
    int status;
    int port = SOCK_PORT;
    char name[33] = HOST_NAME;
    int timeout = SOCK_TIMEOUT;
    unsigned char buf[BUFLEN];

    if ( argc > 1 ) {
	strncpy(name,argv[1],32);
	if ( argc > 2 ) {
	    port = atoi(argv[2]);
	    if ( argc > 3 ) {
		timeout = atoi(argv[3]);
	    }
	}
    }

    //    sock_delay = timeout;
    status = opensock(name,port,&SocketId);

    buf[0] = 0;
    status = writesock(SocketId,buf,1,BINARY);

    return (status);
}

int disconnect (void)
{
    return (closesock(SocketId));
}

int opensock (char *name, int port, int *s)
{
    int status = 0;
    static struct hostent *host;
    static struct sockaddr_in serv_addr;
    static int maxport = 0;
    static int delay = 0, dela;
    static int first = 1;

    if ( maxport == 0 ) {
	maxport = port + PORT_TRY;
	//	delay = sock_delay;
	dela  = 3;
	*s = socket(AF_INET,SOCK_STREAM,0);

	if ( (host = gethostbyname(name))==NULL ) {
	    printf("Could not resolve host '%s'\n", name);
	    return -1;
	}

	serv_addr.sin_family = AF_INET;
	memcpy(&serv_addr.sin_addr.s_addr,host->h_addr,host->h_length);
    }
    
    serv_addr.sin_port = htons(port);

    printf("connecting to host %s %s port %d ... ",host->h_name,
	   inet_ntoa(*(struct in_addr *)host->h_addr),port);
    fflush(stdout);

    //    sock_delay = dela;
    status = connect (*s, (struct sockaddr *)&serv_addr, sizeof(serv_addr));
   
    if(status<0) {
      printf("connection failed!\n");
    }

    //    sock_delay = delay;
    switch (status) {
	case 1 : /* foreign host closed */
		 printf("connection closed!\n");
		 return(3);
	case -1: /* timeout */
		 printf("connection timed out!\n");
		 if ( first ) {
		     first = 0;
//		     port = remstart(name);
		     maxport = port+1;
		     dela = 7;
		 } else {
		     port++;
		 }
		 if ( port < maxport ) {
		     closesock(*s);
		     return ( opensock (name,port,s) );
		 } else {
		     return(5);
		 }
	default: /* normal exit */
		 return(0);
    }

}

int closesock (int s)
{
    return close(s);
}


int readsock (int s, unsigned char *buffer, int len, int mode)
{
  int status;
 
  if ( mode == ASCII ) {
    buffer[0] = 0;
  } else {
    status = read(s,buffer,1);
    if ( status == -1 ) {
      printf("error during read %d\n",errno);
      return(status);
    }
    len = buffer[0];
  }
 
  //  status = read(s,buffer,len);
  status = recv(s,buffer,len,0);
  if ( status == -1 ) {
    printf("error during read %d\n",errno);
  } else {
    status = 0;
    if ( mode == ASCII ) {
      if ( buffer[status] != 10 ) {
        printf("LF not found %d\n",errno);
      }
      buffer[status] = 0;
    }
  }
 
  return(status);

}

int writesock (int s, unsigned char *buffer, int len, int mode)
{
  int lmes,status;
  char buf[BUFLEN+2];
 
  memset(buf,0,sizeof(buf));

  if ( mode == ASCII ) {
    lmes = strlen( buffer ) + 1;
    memcpy(buf,buffer,lmes);
    buf[lmes-1] = 10;
    buf[lmes] = 0;
  } else {
    lmes = len + 1;
    memcpy(&buf[1],buffer,len);
    buf[0] = len;
  }
     
  status = write(s,buf,lmes);
  //  status = send(s,buf,lmes,0);
 
  if ( status == -1 ) {
    printf("error during command send %d\n",errno);
  } else {
    status = 0;
  }
 
  return (status);
}

int decodemsg (unsigned char *buffer, int mode)
{
    int i;

    if ( mode == ASCII ) {
	printf("Executing command %s ...",buffer);
    } else {
	printf("Executing command %d %d",buffer[0],buffer[1]);
	for (i=0; i<6; i++)
	    printf(" %d",buffer[2+i]);
	printf(" ...");
    }
    fflush(stdout);
    sleep(1);
    printf(" done\n");
    return(0);
}

/*******************************************************************/

int openbox(int num)
{
    int status;
    unsigned char buf[BUFLEN];

    buf[0] = 31;
    status = writesock(SocketId,buf,1,BINARY);
    if ( status ) return (status);

    status = readsock(SocketId,buf,BUFLEN,BINARY);
    if ( status ) return (status);
    status = buf[0];
    if ( status ) return (status);

    buf[0] = num;
    status = writesock(SocketId,buf,1,BINARY);
    if ( status ) return (status);

    status = readsock(SocketId,buf,BUFLEN,BINARY);
    if ( status ) return (status);

    status = buf[0];
    return (status);
}

int closebox(int num)
{
    int status;
    unsigned char buf[BUFLEN];

    buf[0] = 32;
    status = writesock(SocketId,buf,1,BINARY);
    if ( status ) return (status);

    status = readsock(SocketId,buf,BUFLEN,BINARY);
    if ( status ) return (status);
    status = buf[0];
    if ( status ) return (status);

    buf[0] = num;
    status = writesock(SocketId,buf,1,BINARY);
    if ( status ) return (status);

    status = readsock(SocketId,buf,BUFLEN,BINARY);
    if ( status ) return (status);

    status = buf[0];
    return (status);
}

int readcmd(unsigned char *buffer, int len, int *time)
{
    int status;
    unsigned char buf[BUFLEN];

    buf[0] = 34;
    status = writesock(SocketId,buf,1,BINARY);
    if ( status ) return (status);

    status = readsock(SocketId,buf,BUFLEN,BINARY);
    if ( status ) return (status);
    status = buf[0];
    if ( status ) return (status);

    buf[0] = *time & 0xFF;
    buf[1] = *time >> 8;
    status = writesock(SocketId,buf,2,BINARY);
    if ( status ) return (status);

    status = readsock(SocketId,buf,BUFLEN,BINARY);
    if ( status ) return (status);
    if ( buf[0] == 0 ) return (ERROR);

/*  buf[1] - not used */
/*  mesnum = buf[2] + buf[3] * 256; */
    *time = buf[4] + buf[5] * 256;
/*  buf[6],buf[7] - not used */

    len = buf[9] + 2;
    memcpy(buffer,&buf[8],len);

    return (status);
}

int writecmd(unsigned char *buffer, int len, int *time)
{
    int status;
    unsigned char buf[BUFLEN];

    buf[0] = 33;
    status = writesock(SocketId,buf,1,BINARY);
    if ( status ) return (status);

    status = readsock(SocketId,buf,BUFLEN,BINARY);
    if ( status ) return (status);
    status = buf[0];
    if ( status ) return (status);

    buf[0] = buffer[0];
    buf[1] = buffer[1];
    buf[2] = *time & 0xFF;
    buf[3] = *time >> 8;
    status = writesock(SocketId,buf,4,BINARY);
    if ( status ) return (status);

    status = readsock(SocketId,buf,BUFLEN,BINARY);
    if ( status ) return (status);
    status = buf[0];
    if ( status ) return (status);

    len = buffer[1];
    if ( len == 0 ) return (status);

    memcpy(buf,&buffer[2],len);
    status = writesock(SocketId,buf,len,BINARY);
    if ( status ) return (status);

    status =  readsock(SocketId,buf,BUFLEN,BINARY);
    if ( status ) return (status);

    status = buf[0];
    return (status);
}


