// le51.cc - test for LE51 miss interface
//
//  Nov-2004 by Solo
//

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>

#include "le51.h"

#define MAX_DATA 20*512

// display MISS raw data
void print_data(short *data, short *nums, int size) 
{
  printf("Addr  Mod Chan Data\n");
  for(int i=0; i<size; i++) printf("%4d %4d %4d 0x%04x\n",
				   i,(nums[i]>>6)&63,nums[i]&63,
				   (unsigned short)data[i]);
}

// display MISS unpacked data (from memory f.ex)
void printu_data(short * data, short *nums, int size)
{
  int det=0;
  int sec=0;
  int words=0;
  int event=0;

  printf("Data in reversed order!\n");
  printf("Addr  Mod Chan Data  Unpack\n");
  for(int i=size-1; i>=0; i--) {
    printf("%4d %4d %4d 0x%04x ",
	   i,(nums[i]>>6)&63,nums[i]&63,
	   (unsigned short)data[i]);

    if (data[i]<0) { // control word
      if (data[i-1]<0) { // memory controller
	det=0; sec=0; words = data[i]&0x3fff;
	printf("AK MEM words=%d\n",words);
      }
      else {
	det = (data[i]>>11)&7; sec=0; words = data[i]&0x7ff;
	printf("AK det=%d words=%d\n",det,words);
      }
    }
    else { // data word
      if ( data[i+1]<0 ) { // control word from AK
	sec = (data[i]>>10)&15;
	event = data[i]&1023;
	printf("AK sec=%d event=%d\n",sec,event);
      }
      else {
	switch (det) {
	case 7: { // LE68 compressed ADC
	  if ((data[i]&0x4000)>0) printf("ADC amp=%d\n",data[i]&4095);
	  else if ((data[i+1]&0x4000)>0) // big amplitude 
	    printf("ADC mod=%d chan=%d\n",(data[i]>>4)&31,data[i]&15);
	  else // small (compressed) amplitude
	    printf("ADC mod=%d chan=%d amp=%d\n",
		   (data[i]>>4)&31,data[i]&15,(data[i]>>9)&31);
	  break;
	}
	default: printf("\n");
	}
      }
    }
  } // end of for
}

int main(int argc, char **argv)
{
  short data[MAX_DATA];
  short nums[MAX_DATA];
  char qname[80];
  int crate=0;
  int n;
  // int module=0;
  //  int address=0;
  //  short d;

  struct timeval tv0, tv1;

  if(argc<4) {
    printf("Usage: le51 qbus crate cmd \n");
    printf("       qbus  = 0-1   - qbus number or /dev/xxx \n");
    printf("       crate = 0-15  - miss crate number \n");
    printf("       cmd   = au    - set autonomous mode \n");
    printf("               z     - clear miss crate \n");
    printf("               s     - get crate status \n");
    printf("               y     - send sync NIM signal \n");
    printf("               d     - read data in PCHI mode \n");
    printf("               b     - read data in PCHI mode by blocks of 16\n");
    printf("               u     - read/unpack data in PCHI mode\n");
    return 0;
  }

  strncpy(qname,argv[1],sizeof(qname));
  if(argv[1][0]=='0') strcpy(qname,"/dev/qbus0");
  if(argv[1][0]=='1') strcpy(qname,"/dev/qbus1");

  sscanf(argv[2],"%d",&crate);
  if(crate<0 || crate>15) { 
    printf("miss crate %d is out of [0-15] range\n",crate); 
    return -1;
  }

  QBUS qb(qname);
  if(qb.node()<0) return -1;

  LE51 mi(&qb, crate);

  // AU
  if(strcmp(argv[3],"au")==0) {
    mi.au();
  }
  // Z
  else if(strcmp(argv[3],"z")==0) {
    mi.Z();
  }
  // S
  else if(strcmp(argv[3],"s")==0) {
    mi.print();
  }
  // Y
  else if(strcmp(argv[3],"y")==0) {
    mi.sync();
  }
  // D
  else if(strcmp(argv[3],"d")==0) {
    gettimeofday(&tv0,0);
    n=mi.sdata(MAX_DATA,data,nums);
    gettimeofday(&tv1,0);
    if(n>0) {
      int usec = (tv1.tv_sec-tv0.tv_sec)*1000000 + tv1.tv_usec - tv0.tv_usec;
      printf("MISS crate %d data %d words %d usec %.1f usec/w:\n",
	     crate,n,usec,(1.0*usec)/n);
      print_data(data, nums, n);
    }
  }
  // B
  else if(strcmp(argv[3],"b")==0) {
    gettimeofday(&tv0,0);
    n=mi.sdata_fast16(MAX_DATA,data,nums);
    gettimeofday(&tv1,0);
    if(n>0) {
      int usec = (tv1.tv_sec-tv0.tv_sec)*1000000 + tv1.tv_usec - tv0.tv_usec;
      printf("MISS crate %d data %d words %d usec %.1f usec/w:\n",
	     crate,n,usec,(1.0*usec)/n);
      print_data(data, nums, n);
    }
  }
  // U
  else if(strcmp(argv[3],"u")==0) {
    n=mi.sdata(MAX_DATA,data,nums);
    if(n>0) {
      printf("MISS crate %d data %d words:\n",crate,n);
      printu_data(data, nums, n);
    }
  }
  // UNKNOWN
  else {
    printf("unknown command: %s\n",argv[3]);
    return -1;
  }

  return 0;
}
