// ROOT macro to check ADC pedestals
// Nov-2004 by Solo

#define LE24_MAX_CHANNELS 18*16;

// global objects
TCanvas *c1;
TCanvas *c2;
TControlBar *mbar;
TControlBar *cbar;
bool Running;

int c_crate;
int c_module;
int c_cycles;
int c_trg;

// start macro
void ped()
{
  // init variables
  c_crate=0;
  c_module=0;
  c_cycles=400;
  c_trg=1;

  // load dialogs
  gROOT->ProcessLine(".L Dialogs.C");

  // start crate selection panel
  crate_panel();
}

// crate selection panel
void crate_panel()
{
  if (cbar != NULL) delete cbar;

  cbar = new TControlBar("vertical", "ADC Crates", 0, 0);
  
  cbar->AddButton("Crate 0 ...","module_panel(0)","Select ADC Crate 0");
  cbar->AddButton("Crate 1 ...","module_panel(1)","Select ADC Crate 1");
  cbar->AddButton("Crate 2 ...","module_panel(2)","Select ADC Crate 2");
  cbar->AddButton("Crate 3 ...","module_panel(3)","Select ADC Crate 3");
  cbar->AddButton("Crate 4 ...","module_panel(4)","Select ADC Crate 4");
  cbar->AddButton("Crate 5 ...","module_panel(5)","Select ADC Crate 5");
  cbar->AddSeparator();
  cbar->AddButton("Cycles...","set_cycles()","Set number of cycles");
  cbar->AddButton("Trigger...","set_trigger()","Set PED or LED trigger");
  cbar->AddButton("&Cancel","Running=false","Cancel current operation");
  cbar->AddButton("&Exit",".q","Exit rootMISS");

  cbar->Show(); 
}

// module selection panel
int module_panel(int crate)
{
  if(crate<0 || crate>5) {
    printf("Crate %d is out of [0,5] range!\n");
    return -1;
  }

  c_crate=crate;
  c_module=0;

  if (mbar!=NULL) delete mbar;

  mbar = new TControlBar("vertical",Form("CR#%d",crate),90,0);

  int max_module=18;
  if(crate==5) max_module=14;

  for(int module=1; module<max_module+1; module++) {
    mbar->AddButton(Form("Module %d",module),Form("get_ped(%d,%d)",crate,module));
  }
  mbar->AddSeparator();
  mbar->AddButton("Update","next_module(0)","Update ped for current module");
  mbar->AddButton("Next","next_module()","Get ped for next module");
  mbar->AddButton("All Modules","all_modules()","Loop through all modules");
  mbar->AddButton("All GIF","all_modules(1)","Make .GIF files for all modules");
  mbar->AddButton("All PS","all_modules(2)","Make .PS files for all modules");
  mbar->AddButton("All in one PS","all_modules(3)","Make one PS file for all modules");
  mbar->AddButton("Cancel","Running=false","Cancel current operation");
  mbar->AddButton("Dismiss","{dismiss(); delete mbar;}","Dismiss this panel");
  mbar->Show();
}

// loop through all modules
void all_modules(int print=0)
{
  c_module=0;

  printf("Looping through all modules in crate %d\n", c_crate);

  if(print==3) c1->Print(Form("c%d.ps[",c_crate)); // just open PS file
  while(next_module()>0) {
    if ( print==1 ) c1->Print(Form("c%d_m%02d.gif",c_crate,c_module));
    else if (print==2) c1->Print(Form("c%d_m%02d.ps",c_crate,c_module));
    else if (print==3) c1->Print(Form("c%d.ps",c_crate));
    gSystem->Sleep(3000); // sleep for 3 seconds
  }
  if(print==3) c1->Print(Form("c%d.ps]",c_crate)); // close PS file
}

// get ped for next module in crate
int next_module(int i=1)
{
  // crate 5 has 14 modules
  if (c_module>17 || (c_crate==5 && c_module>13)) {
    cout << "end of modules for this crate\n"; 
    c_module=0;
    return 0; // no more modules
  }
  c_module+=i;

  get_ped(c_crate,c_module);

  if(!Running) return 0; // was cancelled  
  return 1;
}

// set number of cycles
void set_cycles()
{
  int a = GetIntegerDialog("Set number of cycles to execute", c_cycles);

  if (a==0) return; // cancel button

  if ( a<1 || a>100000 ) {
    printf("Number of cycles %d is out of [1,100000] range\n",a);
    return;
  }

  c_cycles=a;
  printf("Set number of cycles to %d\n",c_cycles);

}

// set type of trigger
void set_trigger()
{
  int a = GetIntegerDialog("Set trigger to pulse, 1=PED, 2=LED 3=LED-PED", c_trg);

  if (a==0) return; // cancel button

  if ( a<1 || a>3 ) {
    printf("Trigger type %d is out of [1,3] range\n",a);
    return;
  }

  c_trg=a;
  printf("Set trigger type to %d\n",c_trg);
}



int get_ped(int crate=0, int module=1)
{
  short ped[LE24_MAX_CHANNELS];
  short num[LE24_MAX_CHANNELS];
  float sum[LE24_MAX_CHANNELS];

  int n, ent;
  char name[10];
  char title[80];

  double mean, rms;

  // MISS stuff
  QBUS *qb0, *qb1;
  LE20C *su;
  LE20I *mi;
  PULSE *pu;
  
  // ROOT stuff
  TH1F *pedH[16]; // array of histograms
  TAxis *axis;    // pointer for axis operations
  Color_t color;  // histogram color

  // check arguments
  if (crate<0 || crate>5) {
    printf("crate %d is out of [0-5] range\n",crate);
    return -1;
  }

  if (module<1 || module>18) {
    printf("module %d is out of [1-18] range\n",module);
    return -1;
  }

  // special case for crate 5
  if (crate==5 & module>14) {
    printf("module %d for crate 5 is out of [1-14] range\n",module);
    return -1;
  }

  // open QBUS devices
  qb0 = new QBUS("/dev/qbus0");
  if (qb0->node()<0) return -1;
  qb1 = new QBUS("/dev/qbus1");
  if (qb1->node()<0) return -1;

  // reset QBUS
  qb0->reset();
  qb1->reset();

  // open SUMMA pulser in crate 4, n=14
  su = new LE20C(qb0, 4);
  pu = new PULSE(su, 14);
  
  // open MISS crate, 0,2,4 - at branch 0 and 1,3,5 at branch 1
  if ( (crate/2)*2 == crate) mi = new LE20I(qb0, crate);
  else mi = new LE20I(qb1, crate);

  if (mi->status()<0) {
    printf("MISS crate %d not found!\n",crate);
    return -1;
  }

  // delete Canvas if exist
  if (c1!=NULL) delete c1;

  // create histograms
  for (int chan=0; chan<16; chan++) {
    if(c_trg==2) sprintf(name,"led%02d",chan+1);
    else if(c_trg==3) sprintf(name,"led-ped%02d",chan+1);
    else sprintf(name,"ped%02d",chan+1);
    sprintf(title,"Ped crate %d module %d chan %d",crate,module,chan+1);
    // check if histogram exists already and delete it
    pedH[chan] = (TH1F*)gDirectory->Get(name);
    if (pedH[chan]!=NULL) delete pedH[chan];

    pedH[chan] = new TH1F(name,title,4096,0,4095);
  }

  printf("Processing crate %d module %d\n",crate,module);

  c_crate=crate;
  c_module=module;

  Running=true;
  int cycle=0;
  int ncycles=0;

  // for LED-PED we need to calculate pedestals
  if (c_trg==3) {
    ncycles=0;
    for(int i=0; i<LE24_MAX_CHANNELS; i++) sum[i]=0.0;
    while(cycle<c_cycles && Running) {
      gSystem->ProcessEvents(); // allow GUI to run

      // clear MISS crate
      mi->au();    // sometimes needed to clear TR
      mi->clear();

      // pulse ped trigger
      pu->ped();

      // read data from sector of LE-19 ADCs
      n = mi->sdata(LE24_MAX_CHANNELS,ped,num);
      if (n<16) printf("Serial data read error, read %d words!\n",n);

      // process data
      if (n>0) {
	for(int i=0; i<n; i++) {
	  if ( (num[i]>>6) == module) sum[i]+=ped[i];
	}
	ncycles++;
      }
      cycle++;

      if(cycle%100 == 0) {
	printf("ped for led cycle=%d\n",cycle);
      }
    }
    for (int i=0; i<LE24_MAX_CHANNELS; i++) sum[i]=sum[i]/ncycles;
  }
  if(!Running) return 0;

  cycle=0;
  ncycles=0;
  // now main cycle 
  while(cycle<c_cycles && Running) {
    gSystem->ProcessEvents(); // allow GUI to run

    // clear MISS crate
    mi->au();    // sometimes needed to clear TR
    mi->clear();

    // pulse ped/led trigger
    if (c_trg==1) pu->ped();
    else pu->led();

    // read data from sector of LE-19 ADCs
    n = mi->sdata(LE24_MAX_CHANNELS,ped,num);
    if (n<16) printf("Serial data read error, read %d words!\n",n);

    // process data
    if (n>0) {
      for(int i=0; i<n; i++) {
	if ( (num[i]>>6) == module) {
	  if(c_trg==3) ped[i]=ped[i]-(short)(sum[i]); // subtract pedestal for led-ped
	  pedH[(num[i]&15)]->Fill(ped[i]);
	}
      }
      ncycles++;
    }
    cycle++;

    if(cycle%100 == 0) {
      if (c_trg==2) printf("led cycle=%d\n",cycle);
      else if (c_trg==3) printf("led-ped cycle=%d\n",cycle); 
      else printf("ped cycle=%d\n",cycle);
    }

  } // end cycle

  // create Canvas
  sprintf(title,"ADC pedestals for crate %d module %d",crate,module);
  c1 = new TCanvas("c1",title,200,0,900,700);
  c1->Divide(4,4);
  
  // increase size of stat box (normal w=0.2 h=0.15)
  gStyle->SetStatW(0.4);
  gStyle->SetStatH(0.35);

  // display data
  for(int chan=0; chan<16; chan++) {
    c1->cd(chan+1);

    // get mean and RMS from the histogram
    mean=pedH[chan]->GetMean();
    rms=pedH[chan]->GetRMS();
    ent=pedH[chan]->GetEntries();
    
    // change fill color according to the signal quality
    color=kGreen;                             // OK
    if (mean<100.0 || rms<0.001) color=kBlack;// too low
    if (mean>300.0 || rms>1.0) color=kBlue;   // average
    if (mean>500.0 || rms>2.0) color=kRed;    // too high 
    if (ent<ncycles) color=kWhite;              // low entries
    if (ent>ncycles) color=kYellow;             // high entries
    pedH[chan]->SetFillColor(color);
    
    // zoom to the mean value
    axis = pedH[chan]->GetXaxis();
    if (c_trg==1) axis->SetRangeUser(mean-10.0, mean+10.0);
    else axis->SetRangeUser(mean-200.0, mean+200.0);


    pedH[chan]->Draw();
    gPad->SetEditable(false); // disable pad edition, so histograms are not movable
  }
  c1->Modified();
  c1->Update();

  // connect event handler to canvas
  c1->Connect("ProcessedEvent(Int_t,Int_t,Int_t,TObject*)", 0, "",
  	      "exec3event(Int_t,Int_t,Int_t,TObject*)");

  printf("Click middle mouse button on a histogram to display it in a separate window\n");

  return 0;
}


// React to event
void exec3event(Int_t event, Int_t x, Int_t y, TObject *selected)
{
   TCanvas *c = (TCanvas *) gTQSender;
   //   printf("Canvas %s: event=%d, x=%d, y=%d, selected=%s\n", c->GetName(),
   //       event, x, y, selected->IsA()->GetName());
   
   // Mouse button 2 up
   if (event == kButton2Up ) {
     TPad *pad = c->GetSelectedPad(); // button 2 down will select the pad
     if (pad) {
       TIter next(pad->GetListOfPrimitives()); // get list of children objects
       TObject *obj;
       while ((obj=next())) {
	 // cout << obj->GetName() << endl;
	 if(obj->InheritsFrom("TH1")) { // if this is a histogram
	   printf("Zooming histo %s\n",obj->GetName());
	   obj->Draw(); // draw in new Canvas
	 }
       } 
     }
   } 
}

// remove canvas
void dismiss()
{
  Running=false;
  if (c2!=NULL) delete c2;
  if (c1!=NULL) delete c1;
}
